package eu.beesoft.abeona.example;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.security.MessageDigest;

/**
 * <p>
 * One possible way to break Abeona license protection is replace original Abeona JAR file by fake
 * JAR with the same name, same classes and methods, but with the different behaviour.
 * </p>
 * <p>
 * This class contains a few methods to protect Abeona JAR file against such attack. All you need to
 * do is to copy these methods into your application code (there are no dependencies except declared
 * imports of standard Java classes). Then you can invoke them when your application starts. Of
 * course, you can rename them to increase the protection.
 * </p>
 * <p>
 * There are implemented two methods to validate Abeona JAR file:
 * </p>
 * <ol>
 * <li>{@link #validateAbeonaJarLocally()} - you can use this method to validate Abeona JAR file if
 * your application has no Internet access. Method has built-in hash codes for all known Abeona
 * releases, so it compares hash code of Abeona JAR file computed in runtime to those patterns.
 * Disadvantage of this method is need to update this method whenever you update Abeona version.</li>
 * <li>{@link #validateAbeonaJarRemote()} - this method is usable when your application has Internet
 * access. It validates Abeona JAR you currently use against hash codes published on www.beesoft.eu.
 * This method will work without any edit with future releases of Abeona.</li>
 * </ol>
 * <p>
 * It is your choice which method you use. Or you can use both depending on the situation.
 * </p>
 * 
 * @author Eugen Kocak
 * 
 */
public class ProtectAbeonaJar {

	// --------------------------------------------- getAbeonaFileName ()
	/**
	 * Returns fully-qualified name of the Abeona JAR file.
	 * 
	 * @return qualified name of the Abeona file (e.g. d:/mylibs/abeona_2011_1.jar)
	 * @throws Exception
	 *             if something goes wrong
	 */
	private String getAbeonaFileName () throws Exception {
		URL url = getClass ().getResource ("/eu/beesoft/abeona/License.class");
		if (url == null) {
			throw new RuntimeException ("Cannot find JAR file for License class");
		}
		String fileName = url.toString ();
		fileName = fileName.substring ("jar:file:/".length (), fileName.lastIndexOf ('!'));
		return fileName;
	}



	// ------------------------------------------- computeSHA256Digest ()
	/**
	 * Computes hash of the file with given name. The hash is computed with SHA-256 algorithm.
	 * 
	 * @param fileName
	 *            - fully qualified name of the file to compute hash
	 * @return computed SHA-256 hash code (as a string of hexa numbers)
	 * @throws Exception
	 *             if something goes wrong
	 */
	private String computeSHA256Digest (String fileName) throws Exception {
		MessageDigest digest = MessageDigest.getInstance ("SHA-256");
		FileInputStream fis = new FileInputStream (fileName);
		byte[] bytes = new byte[1024];
		while (true) {
			int n = fis.read (bytes);
			if (n < 0) {
				break;
			}
			digest.update (bytes, 0, n);
		}
		bytes = digest.digest ();
		StringBuilder sb = new StringBuilder ();
		for (int i = 0; i < bytes.length; i++) {
			sb.append (Integer.toString ((bytes[i] & 0xff) + 0x100, 16).substring (1));
		}
		return sb.toString ().toLowerCase ();
	}



	// -------------------------------------- validateAbeonaJarLocally ()
	/**
	 * Method validates Abeona JAR file locally (without connection to Internet). It needs no
	 * parameters and returns no value. It throws exception, if cannot find Abeona JAR, cannot
	 * compute hash code or this hash code does not meet any of implemented combination Abeona
	 * releases and computed hash codes.
	 * 
	 * @throws Exception
	 *             if Abeona JAR file is not accessible or does not meet implemented hash codes
	 */
	private void validateAbeonaJarLocally () throws Exception {

		// get file name of the currently used Abeona JAR
		String jarFileName = getAbeonaFileName ();

		// compute SHA hash code of the Abeona file
		String sha = computeSHA256Digest (jarFileName);

		// validation for release 2010.1
		if (jarFileName.endsWith ("abeona.jar")) {
			if (sha.equals ("342f6775bcd6ab0278ef80ba7e301b23cb6690726192f41fb4cfff52af550e51")) {
				return;
			}
		}

		// validation for release 2010.2
		else if (jarFileName.endsWith ("abeona_2010_2.jar")) {
			if (sha.equals ("33ce2a722b03cac080cdb4d654b59057593b1d79931997964ae20b8b8348035b")) {
				return;
			}
		}

		// validation for release 2010.3
		else if (jarFileName.endsWith ("abeona_2010_3.jar")) {
			if (sha.equals ("5c195aceecb0b9ec6b24400d5274a652a817cefc23f000bb0ade676790e92783")) {
				return;
			}
		}

		// validation for release 2011.1
		else if (jarFileName.endsWith ("abeona_2011_1.jar")) {
			if (sha.equals ("ac8b11a7cee4ca5195d2fc26f9e7540f659e026d16b6acf7f622ee25a5ca5589")) {
				return;
			}
		}

		// validation for release 2012.1
		else if (jarFileName.endsWith ("abeona_2012_1.jar")) {
			if (sha.equals ("c80e19ad69b997f7f257259b2a36e7b44c086fcfa354f2bd51dfcbbc8fda1297")) {
				return;
			}
		}

		// throw exception if this is unknown combination of the Abeona release and the hash code
		throw new Exception ("Abeona JAR file " + jarFileName + " has bad SHA = " + sha);
	}



	// --------------------------------------- validateAbeonaJarRemote ()
	/**
	 * Method validates Abeona JAR file with connection to Internet. It needs no parameters and
	 * returns no value. It throws exception, if cannot find Abeona JAR, cannot compute hash code or
	 * this hash code does not meet any of implemented combination Abeona releases and computed hash
	 * codes.
	 * 
	 * @throws Exception
	 *             if BeeSoft WEB site is not accessible, if Abeona JAR file is not accessible or
	 *             does not meet implemented hash codes
	 */
	private void validateAbeonaJarRemote () throws Exception {

		// get file name of the currently used Abeona JAR
		String jarFileName = getAbeonaFileName ();

		// compute SHA hash code of the Abeona file
		String sha = computeSHA256Digest (jarFileName);

		// get full name of the .sha file published on the www.beesoft.eu
		String urlName = jarFileName.substring (jarFileName.lastIndexOf ("abeona"));
		urlName = urlName.substring (0, urlName.length () - ".jar".length ());
		urlName = "http://www.beesoft.eu/products/abeona/" + urlName + ".sha";

		// read file and stored SHA hash code from producer site
		URL url = new URL (urlName);
		InputStream is = url.openStream ();
		BufferedReader reader = new BufferedReader (new InputStreamReader (is));
		String originalSHA = reader.readLine ();

		// compare hash codes
		if (sha.equals (originalSHA)) {
			return;
		}

		// throw exception if computed hash code is invalid
		throw new Exception ("Abeona JAR file " + jarFileName + " has bad SHA = " + sha);
	}
}
